@@ -34,12 +34,13 @@
 #include <glom/utils_ui.h>
 #include <glom/glade_utils.h>
 #include <glom/glom_privs.h>
+#include <glom/python_embed/python_ui_callbacks.h>
+#include <glom/python_embed/glom_python.h>
 
 #include <cstdio>
 #include <memory> //For std::auto_ptr<>
 #include <giomm.h>
 #include <sstream> //For stringstream.
-#include <glibmm/i18n.h>
 
 #ifdef GLOM_ENABLE_MAEMO
 #include <hildon/hildon.h>
@@ -61,6 +62,7 @@
 
 #include <gtk/gtkuimanager.h>
 
+#include <glibmm/i18n.h>
 
 namespace Glom
 {
@@ -70,6 +72,9 @@ namespace Glom
 // Global application variable
 Application* global_application = 0;
 
+const char* Application::glade_id("window_main");
+const bool Application::glade_developer(false);
+
 Application::Application(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder)
 : type_base(cobject, "Glom"),
   m_pBoxTop(0),
@@ -87,8 +92,7 @@ Application::Application(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builde
 #endif // !GLOM_ENABLE_CLIENT_ONLY
   m_show_sql_debug(false)
 {
-  // TODO: Use the C++ method when we can use gtkmm 2.18:
-  gtk_window_set_icon_name(GTK_WINDOW(gobj()), "glom");
+  set_icon_name("glom");
 
   //Load widgets from glade file:
   builder->get_widget("bakery_vbox", m_pBoxTop);
@@ -131,6 +135,9 @@ Application::~Application()
   #ifdef GLOM_ENABLE_MAEMO
   m_pFrame->remove_view(&m_appmenu_button_table);
   #endif
+
+  //This was set in the constructor:
+  global_application = 0;
 }
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -171,7 +178,7 @@ bool Application::init(const Glib::ustring& document_uri)
 {
   type_vec_strings vecAuthors;
   vecAuthors.push_back("Murray Cumming <murrayc@murrayc.com>");
-  set_about_information(PACKAGE_VERSION, vecAuthors, _("(C) 2000-2005 Murray Cumming"), _("A Database GUI"));
+  set_about_information(PACKAGE_VERSION, vecAuthors, _("© 2000-2010 Murray Cumming"), _("A Database GUI"));
 
   type_base::init(); //calls init_menus() and init_toolbars()
 
@@ -267,22 +274,25 @@ void Application::init_menus_file()
   m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_File_Open", Gtk::Stock::OPEN),
                         sigc::mem_fun((App_WithDoc&)*this, &App_WithDoc::on_menu_file_open));
 
-  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("BakeryAction_File_SaveAsExample", _("Save As Example"));
+  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("BakeryAction_File_SaveAsExample", _("_Save as Example"));
   m_listDeveloperActions.push_back(action);
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   m_refFileActionGroup->add(action,
                         sigc::mem_fun((Application&)*this, &Application::on_menu_file_save_as_example));
 
-  m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_Menu_File_Export", _("_Export")),
-                        sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_file_export));
+  action = Gtk::Action::create("BakeryAction_Menu_File_Export", _("_Export"));
+  m_refFileActionGroup->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_file_export));
+  m_listTableSensitiveActions.push_back(action);
 
-  action = Gtk::Action::create("BakeryAction_Menu_File_Import", _("Import"));
+  action = Gtk::Action::create("BakeryAction_Menu_File_Import", _("I_mport"));
   m_refFileActionGroup->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_file_import));
+  m_listTableSensitiveActions.push_back(action);
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  m_toggleaction_network_shared = Gtk::ToggleAction::create("BakeryAction_Menu_File_Share", _("Shared On Network"));
+  m_toggleaction_network_shared = Gtk::ToggleAction::create("BakeryAction_Menu_File_Share", _("S_hared on Network"));
   m_refFileActionGroup->add(m_toggleaction_network_shared);
+  m_listTableSensitiveActions.push_back(m_toggleaction_network_shared);
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   m_connection_toggleaction_network_shared =
@@ -291,7 +301,9 @@ void Application::init_menus_file()
   m_listDeveloperActions.push_back(m_toggleaction_network_shared);
 #endif //!GLOM_ENABLE_CLIENT_ONLY
 
-  m_refFileActionGroup->add(Gtk::Action::create("GlomAction_Menu_File_Print", Gtk::Stock::PRINT));
+  action = Gtk::Action::create("GlomAction_Menu_File_Print", Gtk::Stock::PRINT);
+  m_refFileActionGroup->add(action);
+  m_listTableSensitiveActions.push_back(action);
   m_refFileActionGroup->add(Gtk::Action::create("GlomAction_File_Print", _("_Standard")),
                         sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_file_print) );
 
@@ -429,6 +441,7 @@ void Application::init_menus()
   m_refActionGroup_Others->add(action,
                         sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Reports_EditReports) );
   m_listDeveloperActions.push_back(action);
+  m_listTableSensitiveActions.push_back(action);
 #endif
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -436,11 +449,11 @@ void Application::init_menus()
   m_refActionGroup_Others->add(Gtk::Action::create("Glom_Menu_userlevel", _("_User Level")));
   Gtk::RadioAction::Group group_userlevel;
 
-  m_action_menu_userlevel_developer = Gtk::RadioAction::create(group_userlevel, "GlomAction_Menu_userlevel_Developer", _("_Developer"));
+  m_action_menu_userlevel_developer = Gtk::RadioAction::create(group_userlevel, "GlomAction_Menu_userlevel_Developer", C_("User-level menu item", "_Developer"));
   m_refActionGroup_Others->add(m_action_menu_userlevel_developer,
                         sigc::mem_fun(*this, &Application::on_menu_userlevel_developer) );
 
-  m_action_menu_userlevel_operator =  Gtk::RadioAction::create(group_userlevel, "GlomAction_Menu_userlevel_Operator", _("_Operator"));
+  m_action_menu_userlevel_operator =  Gtk::RadioAction::create(group_userlevel, "GlomAction_Menu_userlevel_Operator", C_("User-level menu item", "_Operator"));
   m_refActionGroup_Others->add(m_action_menu_userlevel_operator,
                           sigc::mem_fun(*this, &Application::on_menu_userlevel_operator) );
 #endif // !GLOM_ENABLE_CLIENT_ONLY
@@ -451,16 +464,17 @@ void Application::init_menus()
   Gtk::RadioAction::Group group_mode;
 
   //We remember this action, so that it can be explicitly activated later.
-  m_action_mode_data = Gtk::RadioAction::create(group_mode, "GlomAction_Menu_Mode_Data", _("D_ata"));
+  m_action_mode_data = Gtk::RadioAction::create(group_mode, "GlomAction_Menu_Mode_Data", _("_Data"));
   m_refActionGroup_Others->add(m_action_mode_data,
                         sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Mode_Data) );
 
   m_action_mode_find = Gtk::RadioAction::create(group_mode, "GlomAction_Menu_Mode_Find", _("_Find"));
   m_refActionGroup_Others->add(m_action_mode_find,  Gtk::AccelKey("<control>F"),
                         sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Mode_Find) );
+  m_listTableSensitiveActions.push_back(action);
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  action = Gtk::Action::create("Glom_Menu_Developer", _("_Developer"));
+  action = Gtk::Action::create("Glom_Menu_Developer", C_("Developer menu title", "_Developer"));
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action);
 
@@ -471,14 +485,17 @@ void Application::init_menus()
 
   action = Gtk::Action::create("GlomAction_Menu_Developer_Fields", _("_Fields"));
   m_listDeveloperActions.push_back(action);
+  m_listTableSensitiveActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_fields) );
 
-  action = Gtk::Action::create("GlomAction_Menu_Developer_RelationshipsOverview", _("_Relationships Overview"));
+  action = Gtk::Action::create("GlomAction_Menu_Developer_RelationshipsOverview", _("Relationships _Overview"));
   m_listDeveloperActions.push_back(action);
+  m_listTableSensitiveActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_relationships_overview) );
 
   action = Gtk::Action::create("GlomAction_Menu_Developer_Relationships", _("_Relationships for this Table"));
   m_listDeveloperActions.push_back(action);
+  m_listTableSensitiveActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_relationships) );
 
   m_action_developer_users = Gtk::Action::create("GlomAction_Menu_Developer_Users", _("_Users"));
@@ -487,10 +504,12 @@ void Application::init_menus()
 
   action = Gtk::Action::create("GlomAction_Menu_Developer_PrintLayouts", _("_Print Layouts")); //TODO: Rename? This looks like an action rather than a noun. It won't actually start printing.
   m_listDeveloperActions.push_back(action);
+  m_listTableSensitiveActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_print_layouts));
 
   action = Gtk::Action::create("GlomAction_Menu_Developer_Reports", _("R_eports"));
   m_listDeveloperActions.push_back(action);
+  m_listTableSensitiveActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_reports));
 
   action = Gtk::Action::create("GlomAction_Menu_Developer_Script_Library", _("Script _Library"));
@@ -500,9 +519,10 @@ void Application::init_menus()
 
   action = Gtk::Action::create("GlomAction_Menu_Developer_Layout", _("_Layout"));
   m_listDeveloperActions.push_back(action);
+  m_listTableSensitiveActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_layout));
 
-  action = Gtk::Action::create("GlomAction_Menu_Developer_ChangeLanguage", _("_Test Translation"));
+  action = Gtk::Action::create("GlomAction_Menu_Developer_ChangeLanguage", _("Test Tra_nslation"));
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*this, &Application::on_menu_developer_changelanguage));
 
@@ -597,6 +617,8 @@ void Application::init_menus()
 
   init_menus_help();
 
+  update_table_sensitive_ui();
+
   fill_menu_tables();
 }
 #endif //GLOM_ENABLE_MAEMO
@@ -659,9 +681,19 @@ static bool hostname_is_localhost(const Glib::ustring& hostname)
 
 void Application::ui_warning_load_failed(int failure_code)
 {
-  if(failure_code == Document::LOAD_FAILURE_CODE_FILE_VERSION_TOO_NEW)
+  if(failure_code == Document::LOAD_FAILURE_CODE_NOT_FOUND)
   {
-    ui_warning(_("Open Failed."),
+    //TODO: Put this in the generic bakery code.
+    ui_warning(_("Open Failed"),
+      _("The document could not be found."));
+
+    //TODO: Glom::Bakery::App_WithDoc removes the file from the recent history,
+    //but the initial/welcome dialog doesn't yet update its list when the
+    //recent history changes.
+  }
+  else if(failure_code == Document::LOAD_FAILURE_CODE_FILE_VERSION_TOO_NEW)
+  {
+    ui_warning(_("Open Failed"),
       _("The document could not be opened because it was created or modified by a newer version of Glom."));
   }
   else
@@ -681,11 +713,11 @@ void Application::open_browsed_document(const EpcServiceInfo* server, const Glib
     //Request a password to attempt retrieval of the document over the network:
     Dialog_Connection* dialog_connection = 0;
     //Load the Glade file and instantiate its widgets to get the dialog stuff:
-    Utils::get_glade_widget_derived_with_warning("dialog_connection", dialog_connection);
+    Utils::get_glade_widget_derived_with_warning(dialog_connection);
     dialog_connection->set_transient_for(*this);
     dialog_connection->set_connect_to_browsed();
     dialog_connection->set_database_name(service_name);
-    const int response = Glom::Utils::dialog_run_with_help(dialog_connection, "dialog_connection");
+    const int response = Glom::Utils::dialog_run_with_help(dialog_connection);
     dialog_connection->hide();
     if(response != Gtk::RESPONSE_OK)
       keep_trying = false;
@@ -834,6 +866,9 @@ static bool uri_is_writable(const Glib::RefPtr<const Gio::File>& uri)
 
 Glib::ustring Application::get_file_uri_without_extension(const Glib::ustring& uri)
 {
+  if(uri.empty())
+    return uri;
+
   Glib::RefPtr<Gio::File> file = Gio::File::create_for_uri(uri);
   if(!file)
     return uri; //Actually an error.
@@ -1051,6 +1086,7 @@ bool Application::on_document_load()
       // TODO_clientonly: Tell the user that opening example files is
       // not supported. This could alternatively also be done in
       // Document_after::load_after, I am not sure which is better.
+      ui_warning_load_failed(0);
       return false;
 #endif // GLOM_ENABLE_CLIENT_ONLY
     }
@@ -1177,9 +1213,11 @@ bool Application::on_document_load()
       //Switch to operator mode when opening new documents:
       pDocument->set_userlevel(AppState::USERLEVEL_OPERATOR);
 
+      //Make sure that it's saved in history, even if it was saved from an example file:
+      document_history_add(pDocument->get_file_uri());
+
       //Open default table, or show list of tables instead:
       m_pFrame->do_menu_Navigate_Table(true /* open the default if there is one */);
-
     }
   }
 
@@ -1188,8 +1226,25 @@ bool Application::on_document_load()
 
   update_network_shared_ui();
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
+  //Run any startup script:
+  const Glib::ustring script = pDocument->get_startup_script();
+  if(!script.empty())
+  {
+    Glib::ustring error_message; //TODO: Check this and tell the user.
+    ConnectionPool* connection_pool = ConnectionPool::get_instance();
+    sharedptr<SharedConnection> sharedconnection = connection_pool->connect();
+    AppPythonUICallbacks callbacks;
+    glom_execute_python_function_implementation(script,
+      type_map_fields(), //only used when there is a current table and record.
+      pDocument,
+      Glib::ustring() /* table_name */,
+      sharedptr<Field>(), Gnome::Gda::Value(), // primary key - only used when there is a current table and record.
+      sharedconnection->get_gda_connection(),
+      callbacks,
+      error_message);
+  }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   pDocument->set_allow_autosave(true);
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
@@ -1261,6 +1316,20 @@ void Application::on_userlevel_changed(AppState::userlevels /* userlevel */)
   update_userlevel_ui();
 }
 
+void Application::update_table_sensitive_ui()
+{
+  bool has_table = false;
+
+  if(m_pFrame)
+    has_table = !m_pFrame->get_shown_table_name().empty();
+
+  for(type_listActions::iterator iter = m_listTableSensitiveActions.begin(); iter != m_listTableSensitiveActions.end(); ++iter)
+  {
+    Glib::RefPtr<Gtk::Action> action = *iter;
+    action->set_sensitive(has_table);
+  }
+}
+
 void Application::update_userlevel_ui()
 {
   AppState::userlevels userlevel = get_userlevel();
@@ -1272,6 +1341,9 @@ void Application::update_userlevel_ui()
      action->set_sensitive( userlevel == AppState::USERLEVEL_DEVELOPER );
   }
 
+  //Ensure table sensitive menus stay disabled if necessary.
+  update_table_sensitive_ui();
+
   // Hide users entry from developer menu for connections that don't
   // support users
   if(userlevel == AppState::USERLEVEL_DEVELOPER)
@@ -1314,29 +1386,9 @@ bool Application::offer_new_or_existing()
 {
   //Offer to load an existing document, or start a new one.
   const Glib::ustring glade_path = Utils::get_glade_file_path("glom.glade");
-  Glib::RefPtr<Gtk::Builder> refXml;
-#ifdef GLIBMM_EXCEPTIONS_ENABLED
-  try
-  {
-    refXml = Gtk::Builder::create_from_file(glade_path, "dialog_existing_or_new");
-  }
-  catch(const Glib::Error& ex)
-  {
-    std::cerr << "Application::offer_new_or_existing(): Gtk::Builder::create_from_file() failed: " << ex.what() << std::endl;
-  }
-#else
-  std::auto_ptr<Glib::Error> ex;
-  refXml = Gtk::Builder::create_from_file(glade_path, "dialog_existing_or_new", ex);
-  if(ex.get())
-  {
-    std::cerr << "Application::offer_new_or_existing(): Gtk::Builder::create_from_file() failed:" << ex->what() << std::endl;
-  }
-#endif
-
-  g_assert(refXml);
 
   Dialog_ExistingOrNew* dialog_raw = 0;
-  refXml->get_widget_derived("dialog_existing_or_new", dialog_raw);
+  Utils::get_glade_widget_derived_with_warning(dialog_raw);
   std::auto_ptr<Dialog_ExistingOrNew> dialog(dialog_raw);
   dialog->set_transient_for(*this);
 /*
@@ -1347,7 +1399,7 @@ bool Application::offer_new_or_existing()
   bool ask_again = true;
   while(ask_again)
   {
-    const int response_id = Utils::dialog_run_with_help(dialog.get(), "dialog_existing_or_new");
+    const int response_id = Utils::dialog_run_with_help(dialog_raw);
     dialog->hide();
 
     if(response_id == Gtk::RESPONSE_ACCEPT)
@@ -1390,6 +1442,13 @@ bool Application::offer_new_or_existing()
     {
       return false; //close the window to close the application, because they need to choose a new or existing document.
     }
+    else if((response_id == Gtk::RESPONSE_NONE)
+     || (response_id == 0))
+    {
+       //For instance, the file-open dialog was cancelled after Dialog_ExistingOrNew opened it,
+       //so just ask again.
+       //TODO: Stop Dialog_ExistingOrNew from emitting a response in this case.
+    }
     else
     {
       // This would mean that we got a unhandled response from the dialog
@@ -1470,7 +1529,7 @@ void Application::existing_or_new_new()
         m_pFrame->set_databases_selected(database_name_used);
 
         // Add the document to recent files
-	document_history_add(document->get_file_uri());
+	    document_history_add(document->get_file_uri());
       }
       else
       {
@@ -1590,25 +1649,17 @@ bool Application::recreate_database(bool& user_cancelled)
 
   //Show the user that something is happening, because the INSERTS might take time.
   //TOOD: This doesn't actually show up until near the end, even with Gtk::Main::instance()->iteration().
-  std::auto_ptr<Dialog_ProgressCreating> dialog_progress;
-  Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom.glade"), "window_progress");
-  if(refXml)
-  {
-    Dialog_ProgressCreating* dialog_progress_temp = 0;
-    refXml->get_widget_derived("window_progress", dialog_progress_temp);
-    if(dialog_progress_temp)
-    {
-      dialog_progress_temp->set_message(_("Creating Glom Database"), _("Creating Glom database from example file."));
-      dialog_progress.reset(dialog_progress_temp); //Put the dialog in an auto_ptr so that it will be deleted (and hidden) when the current function returns.
+  Dialog_ProgressCreating* dialog_progress_temp = 0;
+  Utils::get_glade_widget_derived_with_warning(dialog_progress_temp);
+  dialog_progress_temp->set_message(_("Creating Glom Database"), _("Creating Glom database from example file."));
+  std::auto_ptr<Dialog_ProgressCreating> dialog_progress(dialog_progress_temp); //Put the dialog in an auto_ptr so that it will be deleted (and hidden) when the current function returns.
 
-      dialog_progress->set_transient_for(*this);
-      dialog_progress->show();
+  dialog_progress->set_transient_for(*this);
+  dialog_progress->show();
 
-      //Ensure that the dialog is shown, instead of waiting for the application to be idle:
-      while(Gtk::Main::instance()->events_pending())
-        Gtk::Main::instance()->iteration();
-    }
-  }
+  //Ensure that the dialog is shown, instead of waiting for the application to be idle:
+  while(Gtk::Main::instance()->events_pending())
+    Gtk::Main::instance()->iteration();
 
   dialog_progress->pulse();
 
@@ -2370,40 +2421,32 @@ void Application::stop_self_hosting_of_document_database()
 
 void Application::on_menu_developer_changelanguage()
 {
-  Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "dialog_change_language");
-  if(refXml)
+  Dialog_ChangeLanguage* dialog = 0;
+  Utils::get_glade_widget_derived_with_warning(dialog);
+  dialog->set_transient_for(*this);
+  const int response = Glom::Utils::dialog_run_with_help(dialog);
+  dialog->hide();
+
+  if(response == Gtk::RESPONSE_OK)
   {
-    Dialog_ChangeLanguage* dialog = 0;
-    refXml->get_widget_derived("dialog_change_language", dialog);
-    if(dialog)
-    {
-      dialog->set_transient_for(*this);
-      const int response =       Glom::Utils::dialog_run_with_help(dialog, "dialog_change_language");
-      dialog->hide();
+    TranslatableItem::set_current_locale(dialog->get_locale());
 
-      if(response == Gtk::RESPONSE_OK)
-      {
-        TranslatableItem::set_current_locale(dialog->get_locale());
+    //Get the translations from the document (in Operator mode, we only load the necessary translations.)
+    //This also updates the UI, so we show all the translated titles:
+    int failure_code = 0;
+    get_document()->load(failure_code);
 
-       //Get the translations from the document (in Operator mode, we only load the necessary translations.)
-       //This also updates the UI, so we show all the translated titles:
-       int failure_code = 0;
-       get_document()->load(failure_code);
-
-       m_pFrame->show_table_refresh(); //load() doesn't seem to refresh the view.
-      }
-
-      delete dialog;
-    }
+    m_pFrame->show_table_refresh(); //load() doesn't seem to refresh the view.
   }
+
+  delete dialog;
 }
 
 void Application::on_menu_developer_translations()
 {
   if(!m_window_translations)
   {
-    Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(Utils::get_glade_file_path("glom_developer.glade"), "window_translations");
-    refXml->get_widget_derived("window_translations", m_window_translations);
+    Utils::get_glade_widget_derived_with_warning(m_window_translations);
     if(m_window_translations)
     {
       m_pFrame->add_view(m_window_translations);
@@ -2517,7 +2560,9 @@ void Application::update_window_title()
   else //Use the table name if there is no table title.
     table_label = table_name;
 
-  Glib::ustring strTitle = document->get_name() + ": " + table_label;
+  Glib::ustring strTitle = document->get_name();
+  if(!table_label.empty())
+    strTitle += ": " + table_label;
 
   #ifndef GLOM_ENABLE_CLIENT_ONLY
   //Indicate unsaved changes:
