@@ -25,7 +25,7 @@
 #include <gtkmm/entry.h>
 #include <gtkmm/label.h>
 #include <gtkmm/builder.h>
-#include "base_db.h"
+#include <glom/base_db.h>
 #include <libglom/connectionpool.h>
 
 namespace Glom
@@ -36,6 +36,9 @@ class Dialog_Connection
     public Base_DB
 {
 public:
+  static const char* glade_id;
+  static const bool glade_developer;
+
   Dialog_Connection(BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& builder);
   virtual ~Dialog_Connection();
 
