@@ -21,12 +21,9 @@
 #include "flowtable_dnd.h"
 #include "../mode_data/flowtablewithfields.h"
 #include <gtkmm/toolpalette.h>
-#include "placeholder-glom.h"
+#include <glom/mode_data/placeholder-glom.h>
 #include "layouttoolbarbutton.h"
-#include "entryglom.h"
-#include "labelglom.h"
-#include "buttonglom.h"
-#include "imageglom.h"
+#include <glom/mode_data/buttonglom.h>
 
 namespace Glom
 {
